#include <math.h>

#define MAX(a,b) (((a)>(b))?(a):(b))
#define MIN(a,b) (((a)<(b))?(a):(b))

/* Convert color models: HSV to RGB */
/* From Foley, et al */
void
hsv_to_rgb(r, g, b, h, s, v)
float *r, *g, *b; /* red, green, blue in [0,1] */
float h, s, v;    /* hue in [0,360]; saturation, value in [0,1] */
{
  int i;
  float f, p, q, t;

  if(s==0.0) { /* achromatic, hue meaningless */
    (*r) = (*g) = (*b) = v;
  } else {
    if(h==360.0) h=0.0;    /* this is a circle */
    h /= 60.0;             /* h is mostly in one of 6 main hues */
    i=(int)floor(h);       /* i<=h, i is which hue of 0,1,2,3,4,5 */
    f=h-i;                 /* fractional part of h */
    p=v*(1.0-s);           /* purity of this hue */
    q=v*(1.0-(s*f));       /* farness from this hue */
    t=v*(1.0-(s*(1.0-f))); /* closeness to this hue */

    switch(i) {
      case 0: *r=v; *g=t; *b=p; break; /* mostly red */
      case 1: *r=q; *g=v; *b=p; break; /* mostly yellow */
      case 2: *r=p; *g=v; *b=t; break; /* mostly green */
      case 3: *r=p; *g=q; *b=v; break; /* mostly cyan */
      case 4: *r=t; *g=p; *b=v; break; /* mostly blue */
      case 5: *r=v; *g=p; *b=q; break; /* mostly magenta */
    }
  }
}


/* Convert color models: RGB to HSV */
/* From Foley, et al */
void
rgb_to_hsv(r, g, b, h, s, v)
float r, g, b;
float *h, *s, *v;
{
  float maxv, minv;
  float delta;

  maxv=MAX(r, MAX(g, b));
  minv=MIN(r, MIN(g, b));

  *v=maxv;   /* value, v */

  /* Calculate saturation, s */
  if(maxv!=0.0) {
    *s = (maxv-minv)/maxv; /* s is the saturation */
  } else {
    *s = 0.0;   /* saturation is 0 if red, green, and blue are all 0 */
  }

  if(*s==0.0) { /* achromatic (grey, colorless) case */
    *h = -360.0; /* undefined */
  } else { /* chromatic case */
    delta = maxv-minv;

    if(r==maxv) {
      *h = (g-b)/delta; /* resulting color is between yellow and magenta */
    } else if(g==maxv) {
      *h = 2.0 + (b-r)/delta; /* color is between cyan and yellow */
    } else { /* b==maxv */
      *h = 4.0 + (r-g)/delta; /* color is between magenta and cyan */
    }

    *h = *h * 60.0; /* convert hue to degrees */

    if(*h<0.0) {
      *h = *h + 360.0; /* make sure hue is nonnegative */
    }
  } /* chromatic case */
}
